.setcpu "65816"
.autoimport on
.include "inc/common-macros.inc"
.include "inc/GlobalVars.inc"
.include "inc/ports.inc"
.import PaletteBase:far
.import Palette_Field1:far


; API LIST
; =============== Initialization Utilities ===============
.export initSystemGenericStatuses
.export initVideoForGame
.export initVideoForMode7
.export clearVRAMDirect

; ==================== Screen Control ====================
.export disableScreen
.export enableScreensWithBrightness
.export enableBG2WithBrightness
.export setScreenBrightness
.export disableNMI
.export enableNMI
.export forceStopHDMA

.export resetScroll

; ======================= Transfer =======================
.export transferPaletteData
.export transferPaletteDataBulk8
.export transferField1PaletteData
.export transferFontPaletteData
.export transferBitmapDataDMA


;  n[hEFA
; rfI֌W͊Oł̂ŁÂƂpbh͂̃IvV̂
; A16, I16
.proc initSystemGenericStatuses
.a16
.i16
	save_paxy

	set_a8
	.a8
		; Disable old-style pad input
		stz $4016
	set_a16
	.a16

	restore_paxy
	rts
.endproc


;  rfI
; Eʃ[hA`bvTCYݒ
; EVRAMAhXݒ
; A16, I16
.proc initVideoForGame
.a16
.i16
	save_paxy

	; BG configuration
set_a8
.a8
	; $2105:BGMODE
	;    DCBAemmm
	;    DCBA      : BG chr bit size for 
	;                   A=BG1,B=BG2,C=BG3,D=BG4
	;                   if bit 0: 8x8    chr tile
	;                          1: 16x16    chr tile
	;        e     : Mode1 BG3 Priority
	;         mmm  : BG Mode
    ;                 0: Mode0, 1:Mode1, .... 7:Mode7

	; Ƃ肠[h1 BG0:16x16  BG1:8x8 chr tile
	lda #(%00010001) ; order change bit enabled
	sta $2105




	; Tile map address setting for BG1/2/3/4
	; $2107:BG1SC
	; $2108:BG2SC
	; $2109:BG3SC
	; $210a:BG4SC

	;    DCBAemmm
	;    aaaaaayx
	;    aaaaaa    : TileMap VRAM Address( addr>>10 )
	;                  if be wrriten by $14 to reg,  
	;                     is VRAM Addr  0x1400.
	;                           ( 0x400 piece )
	;          y     horizonal mirroring?
	;           x    vertical  mirroring?
	; each max 0x800 bytes size.

	; BG1 Size = 32x32 tiles Map
	;     Map start = +6000W
	; * Tile area = 2048(0x800h) bytes
	lda kMap1StartHi
	sta $2107 ; <- AAAAAASS

	; BG2 Size = 32x32 tiles
	;     Map start = +6800W
	; * Tile area = 2048(0x800h) bytes
	lda kMap2StartHi
	sta $2108

	; BG3 Size = 32x32 tiles
	;     Map start = +7000W
	; * Tile area = 2048(0x800h) bytes
	lda kMap3StartHi
	sta $2109

	; BG4 Size = 32x32 tiles
	;     Map start = +7800W
	; * Tile area = 2048(0x800h) bytes
	lda kMap4StartHi
	sta $210a





	; BG1 and 2 chr address
	; BG3 and 4 chr address
	; $210b:BG12NBA
	; $210c:BG34NBA
	;    bbbbaaaa
	;        aaaa  : Tile character BG1(BG3) 
	;                   VRAM Address( addr>>12 )
	;                  if be wrriten by $X1 to reg,  
	;                     is BG1 VRAM chrAddr  0x1000.
	;                           ( 0x1000 piece )
	;    bbbb        Tile character BG2(BG4) 
	;                   VRAM Address( addr>>12 )
	;                  if be wrriten by $10 to reg,  
	;                     is BG3 VRAM chrAddr  0x1000.
	; each max 0x800 bytes size.


	; BG 1/2 Pattern char VRAM addr Set
	; BG1 : $0000W-$0FFFW (4KB, 256 pettern)
	; BG2 : $1000W-$1FFFW (4KB, 256 pettern)
	lda #$00
	sta $210b

	; BG 3/4 Pattern Source Start
	; BG1 : $2000W-$2FFFW (4KB, 256 pettern)
	; BG2 : $3000W-$3FFFW (4KB, 256 pettern)
	lda #$32
	sta $210c

set_a16
.a16

	restore_paxy
	rts
.endproc


; A16, I16
.proc initVideoForMode7
.a16
.i16
	save_paxy

	; BG configuration
set_a8
.a8

	lda #$00
	sta pM7FillConfig

	lda #$07
	sta $2105 ; <- SSSSPMMM (chip Size, Priority change, screen Mode)

	; init transform
	lda #1
	stz $211B
	sta $211B

	stz $211E
	sta $211E
	; ^^^ note: HIGH BYTE=Int part thus 1.0 = 0100h

	stz $211C
	stz $211C

	stz $211D
	stz $211D

	; BG 1 map area start
	lda #$71  ; map 7000h- , 64x32
	sta $2107 ; <- AAAAAASS

	; BG 1 Pattern Source Start($6000W)
	lda #$06
	sta $210B


set_a16
.a16

	restore_paxy
	rts
.endproc

;  VRAMڃNA
; in A: Dest address(W)
; in Y: Amount
; A16, I16
.proc clearVRAMDirect
.a16
.i16
	save_paxy
	sta pVWriteAddrW

	lp:
		stz pVWriteValW

		dey
		bne lp

	restore_paxy
	rts
.endproc


;  Sʕ\It
; A16, I16
.proc disableScreen
.a16
.i16
	php
	pha

set_a8
.a8
	lda #$80
	sta $2100
set_a16
.a16

	pla
	plp
	rts
.endproc


;  Sʕ\I{Pxݒ
; A16, I16
; in X: brightness(1-15)
.proc enableScreensWithBrightness
.a16
.i16
	save_paxy

set_a8
.a8
	; vvvvv CONFIGURE HERE
	; flags = [X][X][X][SP] [BG4][BG3][BG2][BG1]
;	lda #%00010111 ; BG1+2+3+SP
	lda #%00010001 ; BG1+SP
	sta pScrsEnabled ; Main screen
	stz $212d ; Sub screen (0=all off)

	; brightness
	txa
	and #$0f
	sta $2100

set_a16
.a16
	
	restore_paxy
	rts
.endproc

; A16, I16
; in X: brightness(1-15)
.proc enableBG2WithBrightness
.a16
.i16
	save_paxy

set_a8
.a8
	; vvvvv CONFIGURE HERE
	; flags = [X][X][X][SP] [BG4][BG3][BG2][BG1]
	lda #%00000010 ; BG2
	sta pScrsEnabled ; Main screen
	stz $212d ; Sub screen (0=all off)

	; brightness
	txa
	and #$0f
	sta $2100

set_a16
.a16
	
	restore_paxy
	rts
.endproc

; in A: brightness(0-15)
.proc setScreenBrightness
.a16
.i16
	save_pa

set_a8
.a8
	and #$0f
	sta $2100
set_a16
.a16

	restore_pa
	rts
.endproc


;  NMI(VBLANK荞)L
; A16, I16
.proc enableNMI
.a16
.i16
	pha

	set_a8
	.a8
		lda #$81 ; bit 0 is always on (Joy pad auto read)
		sta $4200
	set_a16
	.a16

	pla
	rts
.endproc

;  NMI(VBLANK荞)~
; A16, I16
.proc disableNMI
.a16
.i16
	pha

	set_a8
	.a8
		lda #$01
		sta $4200
	set_a16
	.a16

	pla
	rts
.endproc


; pbg]
; A16, I16
; in X: Palette select
; in Y: Dest offset (in words)
; Note: each palette entry is 16bit(1W/2B)
.proc transferPaletteData
.a16
.i16
	save_paxy

	; X <- X*32
	txa
	left_shift_5
	tax

	set_a8
	.a8

	; Set write position
	tya
	sta $2121

	ldy #32 ; Loop 32 times (2*16)

:	lda f:PaletteBase, x
	sta $2122

	inx
	dey
	bne :-

	set_a16
	.a16

	restore_paxy
	rts
.endproc

; pbgoN]
; A16, I16
; in X: Palette select
; in Y: Dest offset (in words)
; Note: each palette entry is 16bit(1W/2B)
.proc transferPaletteDataBulk8
.a16
.i16
	save_paxy

	; X <- X*32
	;txa
	;left_shift_5
	;tax

	set_a8
	.a8

	; Set write position
	tya
	sta $2121

	;ldy #32*8 ; Loop 32 times (2*16)
	ldy #2*73

:	lda f:PaletteBase, x
	sta $2122

	inx
	dey
	bne :-

	set_a16
	.a16

	restore_paxy
	rts
.endproc





; pbgoN]@tB[h
; A16, I16
; in X: Palette select
; in Y: Dest offset (in words)
; Note: each palette entry is 16bit(1W/2B)
.proc transferField1PaletteData
.a16
.i16
	save_paxy

	; X <- X*32
	;txa
	;left_shift_5
	;tax

	set_a8
	.a8

	; Set write position
	tya
	sta $2121

	;ldy #32*8 ; Loop 32 times (2*16)
	ldy #2*10

:	lda f:Palette_Field1, x
	sta $2122

	inx
	dey
	bne :-

	set_a16
	.a16

	restore_paxy
	rts
.endproc






; tHgppbg㏑oN]
; A16, I16
; Note: each palette entry is 16bit(1W/2B)
.proc transferFontPaletteData
.a16
.i16
	save_paxy

	set_a8
	.a8

	; Set write position
	stz $2121

	; pbgOF
	stz $2122
	stz $2122

	; pbgPF
	lda	#$ff
	sta $2122
	lda	#$7f
	sta $2122

	; pbgQFDF( 80,80,80 )
	lda	#$10
	sta $2122
	lda	#$42
	sta $2122

	; pbgRFF( 00,ff,ff )
	lda	#$e0
	sta $2122
	lda	#$7f
	sta $2122

	; pbgSF
	stz $2122
	stz $2122

	; pbgTF
	lda	#$1f
	sta $2122
	lda	#$0
	sta $2122

	; pbg6FԂ̊DF( 00,00,80 )
	lda	#$10
	sta $2122
	lda	#$42
	sta $2122


	; pbg7FF( 00,ff,ff )
	lda	#$e0
	sta $2122
	lda	#$7f
	sta $2122






	; XvCgppbg
	lda #$80
	sta $2121

	; pbgOF
	stz $2122
	stz $2122

	; pbgPF
	lda	#$ff
	sta $2122
	lda	#$7f
	sta $2122

	; pbgQFDF( 80,80,80 )
	lda	#$10
	sta $2122
	lda	#$42
	sta $2122

	; pbgRFF( 00,ff,ff )
	lda	#$e0
	sta $2122
	lda	#$7f
	sta $2122


	; pbgԍ0x84( 128+4) `0x94 
	; ̊Ԃ̐FQƂĂHHHH
	; ȂŁH
	; chr:5555,0000 ̗񂾂ƁEEE
	; F4bit QƂƂāA
	; 0x80+15 ̂mB

	; 8x8bit
	; 7 6 5 4 3 2 1 0
	; LINE0D7-D0 ̃oCgAFŉʃrbgH
	; LINE1D7-D0 ̃oCgAFŉʃrbgH
	; .........
	; LINE7D7-D0 ̃oCgAFŉʃrbgH
	; LINE0-7D7-D0 ̃oCgAFD1rbgH
	; LINE0-7D7-D0 ̃oCgAFD1rbgH
	; .........
	; LINE7D7-D0 ̃oCgAFD1rbgH
	;
	; EEEƂŐFƂ̃p^[f[^͂

.if 0

	lda #$8d
	sta $2121

	; PR΂߂̂
	lda	#$0f
	sta $2122
	lda	#$0f
	sta $2122


	; PS΂߂̂
	lda	#$00
	sta $2122
	lda	#$0f
	sta $2122

	; PT΂߂̂
	lda	#$ff
	sta $2122
	lda	#$7f
	sta $2122

	ldy		#$0c
:
	lda	#$ff
	sta $2122
	lda	#$7f
	sta $2122
	dey
	bne	:-

.endif


	set_a16
	.a16

	restore_paxy
	rts
.endproc




; VRAMf[^DMA]
; A16, I16
; in X: Dest word address
; in Y: Size (in BYTES)
.proc transferBitmapDataDMA
.a16
.i16
	save_paxy

	; Set dest VRAM address
	stx $2116

	; Set length(bytes)
	sty pDMA0ByteCountW

	; load address(long)
	lda gAssetBankTemp
	ldy gAssetOffsetTemp

	set_a8
	.a8
		; Set DMA source address
		sta pDMA0SourceBank    ; bank (8bit)
		sty pDMA0SourceOffsetW ; offset(16bit)

		; Set DMA target
		; write to $2118(VRAM channel)
		lda #$18
		sta pDMA0Dest

		; Write a word
		lda #$01
		sta pDMA0Config

		; Start - - - - - - - - - - - - -
		lda #$01
		sta pDMATrigger

	set_a16
	.a16

	restore_paxy
	rts
.endproc


; XN[ʒu
; A16, I16
.proc resetScroll
.a16
.i16
	save_pa

	; BGɃXN[Kp
set_a8
.a8
	; BG1HOFS  Horizontal Scroll
	;   M7HOFS -  Mode 7 BG Horizontal Scroll
	; BG1VOFS  BG1 Vertical Scroll
	;   M7VOFS -  Mode 7 BG Vertical Scroll

	; $210d:BG1HOFS,M7HOFS
	; $210e:BG1VOFS,M7VOFS
	;    ------xx  xxxxxxxx
	;        10 bit 

	; FRONT
	lda #$00
	sta pScrollX0
	sta pScrollX0

	sta pScrollY0
	sta pScrollY0

	; BACK
	sta pScrollX1
	sta pScrollX1

	sta pScrollY1
	sta pScrollY1

	; TEXT
	sta pScrollX2
	sta pScrollX2

	sta pScrollY2
	sta pScrollY2


	sta pScrollX3
	sta pScrollX3
	sta pScrollY3
	sta pScrollY3


set_a16
.a16

	restore_pa
	rts
.endproc

; A16, I16
.proc forceStopHDMA
.a16
.i16
	save_pa

set_a8
.a8
	stz pHDMATrigger
set_a16
.a16

	restore_pa
	rts
.endproc
